==> pyport-0.2.3.tar.gz: pyport-0.2.3/pyport/logging.py <==
"""
Logging utilities for the PyPort client library.

This module provides utilities for configuring and using logging in the PyPort client library.
It includes functions for setting up logging, masking sensitive information, and generating
correlation IDs for tracking requests across systems.
"""
import logging
import uuid
import json
import re
from typing import Any, Dict, Optional, Set

# Default logger for the PyPort client library
logger = logging.getLogger("pyport")

# Mapping of string log levels to logging module constants
LOG_LEVEL_MAP = {
    "DEBUG": logging.DEBUG,
    "INFO": logging.INFO,
    "WARNING": logging.WARNING,
    "ERROR": logging.ERROR,
    "CRITICAL": logging.CRITICAL,
}

# Sensitive fields that should be masked in logs
SENSITIVE_FIELDS = {
    "client_id", "client_secret", "token", "accessToken", "refreshToken", "password",
    "secret", "api_key", "apiKey", "Authorization", "auth", "credentials"
}

# Regular expression for masking sensitive values in URLs (e.g., tokens in query parameters)
URL_SENSITIVE_PATTERN = re.compile(
    r"(token|key|secret|password|credential|auth)=([^&]+)",
    re.IGNORECASE
)


def init_logging(log_level: str) -> None:
    """
    Initialize logging with a string log level.

    This is a simplified version of configure_logging that takes a string log level
    and sets up basic logging with a file and stream handler.

    Args:
        log_level: The logging level as a string ("DEBUG", "INFO", etc.)
    """
    level = LOG_LEVEL_MAP.get(log_level.upper(), logging.INFO)
    logging.basicConfig(level=level,
                        format='%(asctime)s - %(levelname)s - %(message)s',
                        handlers=[
                            logging.FileHandler("app.log"),
                            logging.StreamHandler()
                        ])


def configure_logging(
    level: int = logging.INFO,
    format_string: Optional[str] = None,
    handler: Optional[logging.Handler] = None,
    propagate: bool = False,
    log_file: Optional[str] = None
) -> None:
    """
    Configure logging for the PyPort client library.

    Args:
        level: The logging level to use (default: logging.INFO).
        format_string: The format string to use for log messages.
            If None, a default format will be used.
        handler: A logging handler to use. If None, a StreamHandler will be created.
        propagate: Whether to propagate log messages to the root logger (default: False).
        log_file: Path to a log file. If provided, a FileHandler will be added.
    """
    global logger

    # Set the logging level
    logger.setLevel(level)

    # Set propagation
    logger.propagate = propagate

    # Remove any existing handlers
    for hdlr in logger.handlers[:]:
        logger.removeHandler(hdlr)

    # Create a formatter
    if format_string is None:
        format_string = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    formatter = logging.Formatter(format_string)

    # Create a handler if none was provided
    if handler is None:
        handler = logging.StreamHandler()

    # Set the formatter and add the handler
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    # Add a file handler if a log file was specified
    if log_file:
        file_handler = logging.FileHandler(log_file)
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)


def get_correlation_id() -> str:
    """
    Generate a unique correlation ID for tracking requests.

    Returns:
        A unique correlation ID string.
    """
    return str(uuid.uuid4())


def mask_sensitive_data(data: Any, sensitive_fields: Optional[Set[str]] = None) -> Any:
    """
    Mask sensitive data in logs.

    This function recursively traverses dictionaries and lists, masking values for keys
    that match sensitive field names.

    Args:
        data: The data to mask.
        sensitive_fields: A set of field names to mask. If None, a default set will be used.

    Returns:
        The masked data.
    """
    if sensitive_fields is None:
        sensitive_fields = SENSITIVE_FIELDS

    if isinstance(data, dict):
        masked_data = {}
        for key, value in data.items():
            if key.lower() in {field.lower() for field in sensitive_fields}:
                # Mask the value, preserving the first and last character if it's a string
                if isinstance(value, str) and len(value) > 6:
                    masked_data[key] = f"{value[0]}{'*' * (len(value) - 2)}{value[-1]}"
                else:
                    masked_data[key] = "********"
            elif isinstance(value, (dict, list)):
                masked_data[key] = mask_sensitive_data(value, sensitive_fields)
            else:
                masked_data[key] = value
        return masked_data
    elif isinstance(data, list):
        return [mask_sensitive_data(item, sensitive_fields) for item in data]
    else:
        return data


def mask_url(url: str) -> str:
    """
    Mask sensitive information in URLs.

    Args:
        url: The URL to mask.

    Returns:
        The masked URL.
    """
    # Handle mock objects in tests
    if not isinstance(url, str):
        return str(url)

    # Mask sensitive query parameters
    return URL_SENSITIVE_PATTERN.sub(r"\1=********", url)


def format_request_for_logging(
    method: str,
    url: str,
    headers: Optional[Dict[str, str]] = None,
    params: Optional[Dict[str, Any]] = None,
    data: Optional[Any] = None,
    json_data: Optional[Any] = None,
    correlation_id: Optional[str] = None,
    **kwargs
) -> Dict[str, Any]:
    """
    Format a request for logging, masking sensitive information.

    Args:
        method: The HTTP method.
        url: The URL.
        headers: The request headers.
        params: The query parameters.
        data: The request body data.
        json_data: The request body JSON data.
        correlation_id: A correlation ID for tracking the request.
        **kwargs: Additional request parameters.

    Returns:
        A dictionary containing the formatted request information.
    """
    # Create a dictionary with request information
    request_info = {
        "method": method,
        "url": mask_url(url),
        "correlation_id": correlation_id or get_correlation_id()
    }

    # Add headers if present, masking sensitive information
    if headers:
        request_info["headers"] = mask_sensitive_data(headers)

    # Add query parameters if present
    if params:
        request_info["params"] = mask_sensitive_data(params)

    # Add request body if present
    if data:
        if isinstance(data, (dict, list)):
            request_info["data"] = mask_sensitive_data(data)
        else:
            request_info["data"] = "<<binary data>>"

    # Add JSON data if present
    if json_data:
        request_info["json"] = mask_sensitive_data(json_data)

    return request_info


def format_response_for_logging(
    response,
    correlation_id: Optional[str] = None,
    include_body: bool = True
) -> Dict[str, Any]:
    """
    Format a response for logging, masking sensitive information.

    Args:
        response: The response object.
        correlation_id: A correlation ID for tracking the request.
        include_body: Whether to include the response body in the log.

    Returns:
        A dictionary containing the formatted response information.
    """
    # Handle mock objects in tests
    if hasattr(response, "__class__") and "MagicMock" in response.__class__.__name__:
        return {
            "status_code": getattr(response, "status_code", 200),
            "url": str(getattr(response, "url", "mock-url")),
            "correlation_id": correlation_id,
            "elapsed": 0.0,
            "mock": True
        }

    # Create a dictionary with response information
    response_info = {
        "status_code": response.status_code,
        "url": mask_url(response.url),
        "correlation_id": correlation_id,
        "elapsed": response.elapsed.total_seconds()
    }

    # Add headers if present
    if response.headers:
        response_info["headers"] = mask_sensitive_data(dict(response.headers))

    # Add response body if requested
    if include_body and response.content:
        try:
            # Try to parse as JSON
            body = response.json()
            response_info["body"] = mask_sensitive_data(body)
        except (ValueError, json.JSONDecodeError):
            # If not JSON, include the content length
            response_info["content_length"] = len(response.content)

            # Include a preview of text content if it's not too large
            if len(response.content) < 1000 and response.headers.get("content-type", "").startswith("text/"):
                response_info["body_preview"] = response.text[:500] + "..." if len(response.text) > 500 else response.text

    return response_info


def log_request(
    method: str,
    url: str,
    headers: Optional[Dict[str, str]] = None,
    params: Optional[Dict[str, Any]] = None,
    data: Optional[Any] = None,
    json_data: Optional[Any] = None,
    correlation_id: Optional[str] = None,
    **kwargs
) -> str:
    """
    Log a request, masking sensitive information.

    Args:
        method: The HTTP method.
        url: The URL.
        headers: The request headers.
        params: The query parameters.
        data: The request body data.
        json_data: The request body JSON data.
        correlation_id: A correlation ID for tracking the request.
        **kwargs: Additional request parameters.

    Returns:
        The correlation ID used for the request.
    """
    # Generate a correlation ID if none was provided
    if correlation_id is None:
        correlation_id = get_correlation_id()

    # Only format and log the request if debug logging is enabled
    if logger.isEnabledFor(logging.DEBUG):
        # Format the request for logging
        request_info = format_request_for_logging(
            method, url, headers, params, data, json_data, correlation_id, **kwargs
        )

        # Log the request
        logger.debug(f"Request: {json.dumps(request_info)}")

    return correlation_id


def log_response(
    response,
    correlation_id: Optional[str] = None,
    include_body: bool = True
) -> None:
    """
    Log a response, masking sensitive information.

    Args:
        response: The response object.
        correlation_id: A correlation ID for tracking the request.
        include_body: Whether to include the response body in the log.
    """
    # Only format and log the response if debug logging is enabled
    if logger.isEnabledFor(logging.DEBUG):
        # Format the response for logging
        response_info = format_response_for_logging(response, correlation_id, include_body)

        # Log the response
        logger.debug(f"Response: {json.dumps(response_info)}")


def log_error(
    error: Exception,
    correlation_id: Optional[str] = None
) -> None:
    """
    Log an error, masking sensitive information.

    Args:
        error: The error to log.
        correlation_id: A correlation ID for tracking the request.
    """
    # Only create the error info if error logging is enabled
    if logger.isEnabledFor(logging.ERROR):
        # Create a dictionary with error information
        error_info = {
            "type": type(error).__name__,
            "message": str(error),
            "correlation_id": correlation_id
        }

        # Add additional information for Port API errors
        if hasattr(error, "status_code"):
            error_info["status_code"] = error.status_code
        if hasattr(error, "endpoint"):
            error_info["endpoint"] = error.endpoint
        if hasattr(error, "method"):
            error_info["method"] = error.method
        if hasattr(error, "response_body") and error.response_body:
            error_info["response_body"] = mask_sensitive_data(error.response_body)

        # Log the error
        logger.error(f"Error: {json.dumps(error_info)}")

==> pyport-0.3.3.tar.gz: pyport-0.3.3/pyport/logging.py <==
"""
Logging utilities for the PyPort client library.

This module provides utilities for configuring and using logging in the PyPort client library.
It includes functions for setting up logging, masking sensitive information, and generating
correlation IDs for tracking requests across systems.
"""
import logging
import uuid
import json
import re
from typing import Any, Dict, Optional, Set

# Default logger for the PyPort client library
logger = logging.getLogger("pyport")

# Mapping of string log levels to logging module constants
LOG_LEVEL_MAP = {
    "DEBUG": logging.DEBUG,
    "INFO": logging.INFO,
    "WARNING": logging.WARNING,
    "ERROR": logging.ERROR,
    "CRITICAL": logging.CRITICAL,
}

# Sensitive fields that should be masked in logs
SENSITIVE_FIELDS = {
    "client_id", "client_secret", "token", "accessToken", "refreshToken", "password",
    "secret", "api_key", "apiKey", "Authorization", "auth", "credentials"
}

# Regular expression for masking sensitive values in URLs (e.g., tokens in query parameters)
URL_SENSITIVE_PATTERN = re.compile(
    r"(token|key|secret|password|credential|auth)=([^&]+)",
    re.IGNORECASE
)


def init_logging(log_level: str) -> None:
    """
    Initialize logging with a string log level.

    This is a simplified version of configure_logging that takes a string log level
    and sets up basic logging with a file and stream handler.

    Args:
        log_level: The logging level as a string ("DEBUG", "INFO", etc.)
    """
    level = LOG_LEVEL_MAP.get(log_level.upper(), logging.INFO)
    logging.basicConfig(level=level,
                        format='%(asctime)s - %(levelname)s - %(message)s',
                        handlers=[
                            logging.FileHandler("app.log"),
                            logging.StreamHandler()
                        ])


def configure_logging(
    level: int = logging.INFO,
    format_string: Optional[str] = None,
    handler: Optional[logging.Handler] = None,
    propagate: bool = False,
    log_file: Optional[str] = None
) -> None:
    """
    Configure logging for the PyPort client library.

    Args:
        level: The logging level to use (default: logging.INFO).
        format_string: The format string to use for log messages.
            If None, a default format will be used.
        handler: A logging handler to use. If None, a StreamHandler will be created.
        propagate: Whether to propagate log messages to the root logger (default: False).
        log_file: Path to a log file. If provided, a FileHandler will be added.
    """
    # Access the logger defined at module level

    # Set the logging level
    logger.setLevel(level)

    # Set propagation
    logger.propagate = propagate

    # Remove any existing handlers
    for hdlr in logger.handlers[:]:
        logger.removeHandler(hdlr)

    # Create a formatter
    if format_string is None:
        format_string = "%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    formatter = logging.Formatter(format_string)

    # Create a handler if none was provided
    if handler is None:
        handler = logging.StreamHandler()

    # Set the formatter and add the handler
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    # Add a file handler if a log file was specified
    if log_file:
        file_handler = logging.FileHandler(log_file)
        file_handler.setFormatter(formatter)
        logger.addHandler(file_handler)


def get_correlation_id() -> str:
    """
    Generate a unique correlation ID for tracking requests.

    Returns:
        A unique correlation ID string.
    """
    return str(uuid.uuid4())


def mask_sensitive_data(data: Any, sensitive_fields: Optional[Set[str]] = None) -> Any:
    """
    Mask sensitive data in logs.

    This function recursively traverses dictionaries and lists, masking values for keys
    that match sensitive field names.

    Args:
        data: The data to mask.
        sensitive_fields: A set of field names to mask. If None, a default set will be used.

    Returns:
        The masked data.
    """
    if sensitive_fields is None:
        sensitive_fields = SENSITIVE_FIELDS

    if isinstance(data, dict):
        masked_data = {}
        for key, value in data.items():
            if key.lower() in {field.lower() for field in sensitive_fields}:
                # Mask the value, preserving the first and last character if it's a string
                if isinstance(value, str) and len(value) > 6:
                    masked_data[key] = f"{value[0]}{'*' * (len(value) - 2)}{value[-1]}"
                else:
                    masked_data[key] = "********"
            elif isinstance(value, (dict, list)):
                masked_data[key] = mask_sensitive_data(value, sensitive_fields)
            else:
                masked_data[key] = value
        return masked_data
    elif isinstance(data, list):
        return [mask_sensitive_data(item, sensitive_fields) for item in data]
    else:
        return data


def mask_url(url: str) -> str:
    """
    Mask sensitive information in URLs.

    Args:
        url: The URL to mask.

    Returns:
        The masked URL.
    """
    # Handle mock objects in tests
    if not isinstance(url, str):
        return str(url)

    # Mask sensitive query parameters
    return URL_SENSITIVE_PATTERN.sub(r"\1=********", url)


def format_request_for_logging(
    method: str,
    url: str,
    headers: Optional[Dict[str, str]] = None,
    params: Optional[Dict[str, Any]] = None,
    data: Optional[Any] = None,
    json_data: Optional[Any] = None,
    correlation_id: Optional[str] = None,
    **kwargs
) -> Dict[str, Any]:
    """
    Format a request for logging, masking sensitive information.

    Args:
        method: The HTTP method.
        url: The URL.
        headers: The request headers.
        params: The query parameters.
        data: The request body data.
        json_data: The request body JSON data.
        correlation_id: A correlation ID for tracking the request.
        **kwargs: Additional request parameters.

    Returns:
        A dictionary containing the formatted request information.
    """
    # Create a dictionary with request information
    request_info = {
        "method": method,
        "url": mask_url(url),
        "correlation_id": correlation_id or get_correlation_id()
    }

    # Add headers if present, masking sensitive information
    if headers:
        request_info["headers"] = mask_sensitive_data(headers)

    # Add query parameters if present
    if params:
        request_info["params"] = mask_sensitive_data(params)

    # Add request body if present
    if data:
        if isinstance(data, (dict, list)):
            request_info["data"] = mask_sensitive_data(data)
        else:
            request_info["data"] = "<<binary data>>"

    # Add JSON data if present
    if json_data:
        request_info["json"] = mask_sensitive_data(json_data)

    return request_info


def format_response_for_logging(
    response,
    correlation_id: Optional[str] = None,
    include_body: bool = True
) -> Dict[str, Any]:
    """
    Format a response for logging, masking sensitive information.

    Args:
        response: The response object.
        correlation_id: A correlation ID for tracking the request.
        include_body: Whether to include the response body in the log.

    Returns:
        A dictionary containing the formatted response information.
    """
    # Handle mock objects in tests
    if hasattr(response, "__class__") and "MagicMock" in response.__class__.__name__:
        return {
            "status_code": getattr(response, "status_code", 200),
            "url": str(getattr(response, "url", "mock-url")),
            "correlation_id": correlation_id,
            "elapsed": 0.0,
            "mock": True
        }

    # Create a dictionary with response information
    response_info = {
        "status_code": response.status_code,
        "url": mask_url(response.url),
        "correlation_id": correlation_id,
        "elapsed": response.elapsed.total_seconds()
    }

    # Add headers if present
    if response.headers:
        response_info["headers"] = mask_sensitive_data(dict(response.headers))

    # Add response body if requested
    if include_body and response.content:
        try:
            # Try to parse as JSON
            body = response.json()
            response_info["body"] = mask_sensitive_data(body)
        except (ValueError, json.JSONDecodeError):
            # If not JSON, include the content length
            response_info["content_length"] = len(response.content)

            # Include a preview of text content if it's not too large
            content_type = response.headers.get("content-type", "")
            if len(response.content) < 1000 and content_type.startswith("text/"):
                preview = response.text[:500]
                if len(response.text) > 500:
                    preview += "..."
                response_info["body_preview"] = preview

    return response_info


def log_request(
    method: str,
    url: str,
    headers: Optional[Dict[str, str]] = None,
    params: Optional[Dict[str, Any]] = None,
    data: Optional[Any] = None,
    json_data: Optional[Any] = None,
    correlation_id: Optional[str] = None,
    **kwargs
) -> str:
    """
    Log a request, masking sensitive information.

    Args:
        method: The HTTP method.
        url: The URL.
        headers: The request headers.
        params: The query parameters.
        data: The request body data.
        json_data: The request body JSON data.
        correlation_id: A correlation ID for tracking the request.
        **kwargs: Additional request parameters.

    Returns:
        The correlation ID used for the request.
    """
    # Generate a correlation ID if none was provided
    if correlation_id is None:
        correlation_id = get_correlation_id()

    # Only format and log the request if debug logging is enabled
    if logger.isEnabledFor(logging.DEBUG):
        # Format the request for logging
        request_info = format_request_for_logging(
            method, url, headers, params, data, json_data, correlation_id, **kwargs
        )

        # Log the request
        logger.debug(f"Request: {json.dumps(request_info)}")

    return correlation_id


def log_response(
    response,
    correlation_id: Optional[str] = None,
    include_body: bool = True
) -> None:
    """
    Log a response, masking sensitive information.

    Args:
        response: The response object.
        correlation_id: A correlation ID for tracking the request.
        include_body: Whether to include the response body in the log.
    """
    # Only format and log the response if debug logging is enabled
    if logger.isEnabledFor(logging.DEBUG):
        # Format the response for logging
        response_info = format_response_for_logging(response, correlation_id, include_body)

        # Log the response
        logger.debug(f"Response: {json.dumps(response_info)}")


def log_error(
    error: Exception,
    correlation_id: Optional[str] = None
) -> None:
    """
    Log an error, masking sensitive information.

    Args:
        error: The error to log.
        correlation_id: A correlation ID for tracking the request.
    """
    # Only create the error info if error logging is enabled
    if logger.isEnabledFor(logging.ERROR):
        # Create a dictionary with error information
        error_info = {
            "type": type(error).__name__,
            "message": str(error),
            "correlation_id": correlation_id
        }

        # Add additional information for Port API errors
        if hasattr(error, "status_code"):
            error_info["status_code"] = error.status_code
        if hasattr(error, "endpoint"):
            error_info["endpoint"] = error.endpoint
        if hasattr(error, "method"):
            error_info["method"] = error.method
        if hasattr(error, "response_body") and error.response_body:
            error_info["response_body"] = mask_sensitive_data(error.response_body)

        # Log the error
        logger.error(f"Error: {json.dumps(error_info)}")
